@@ -16,35 +16,34 @@
 
 package org.gradle.api.internal.changedetection;
 
+import org.gradle.CacheUsage;
 import org.gradle.api.DefaultTask;
 import org.gradle.api.internal.TaskInternal;
 import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.invocation.Gradle;
-import org.gradle.cache.*;
-import org.gradle.util.*;
+import org.gradle.cache.CacheRepository;
+import org.gradle.cache.internal.DefaultCacheRepository;
+import org.gradle.internal.id.RandomLongIdGenerator;
+import org.gradle.test.fixtures.file.TestFile;
+import org.gradle.test.fixtures.file.TestNameTestDirectoryProvider;
+import org.gradle.testfixtures.internal.InMemoryCacheFactory;
+import org.gradle.util.HelperUtil;
 import org.hamcrest.Matcher;
-import org.jmock.Expectations;
-import org.jmock.integration.junit4.JMock;
-import org.jmock.integration.junit4.JUnit4Mockery;
 import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
-import org.junit.runner.RunWith;
 
 import java.io.File;
 import java.util.*;
 
-import static org.gradle.util.Matchers.*;
+import static org.gradle.util.Matchers.isEmpty;
 import static org.gradle.util.WrapUtil.*;
 import static org.hamcrest.Matchers.*;
 import static org.junit.Assert.*;
 
-@RunWith(JMock.class)
 public class DefaultTaskArtifactStateRepositoryTest {
     @Rule
-    public TemporaryFolder tmpDir = new TemporaryFolder();
-    private final JUnit4Mockery context = new JUnit4GroovyMockery();
-    private final CacheRepository cacheRepository = context.mock(CacheRepository.class);
+    public TestNameTestDirectoryProvider tmpDir = new TestNameTestDirectoryProvider();
     private final ProjectInternal project = HelperUtil.createRootProject();
     private final Gradle gradle = project.getGradle();
     private final TestFile outputFile = tmpDir.file("output-file");
@@ -60,33 +59,20 @@ public class DefaultTaskArtifactStateRepositoryTest {
     private final Set<TestFile> inputFiles = toSet(inputFile, inputDir, missingInputFile);
     private final Set<TestFile> outputFiles = toSet(outputFile, outputDir, emptyOutputDir, missingOutputFile);
     private final Set<TestFile> createFiles = toSet(outputFile, outputDirFile, outputDirFile2);
-    private final PersistentCache persistentCache = context.mock(PersistentCache.class);
     private DefaultTaskArtifactStateRepository repository;
 
     @Before
     public void setup() {
-        context.checking(new Expectations(){{
-            CacheBuilder builder = context.mock(CacheBuilder.class);
-
-            one(cacheRepository).cache("outputFileStates");
-            will(returnValue(builder));
-
-            one(builder).open();
-            will(returnValue(persistentCache));
-
-            one(persistentCache).openIndexedCache();
-            will(returnValue(new TestIndexedCache()));
-        }});
-
+        CacheRepository cacheRepository = new DefaultCacheRepository(tmpDir.createDir("user-home"), null, CacheUsage.ON, new InMemoryCacheFactory());
+        TaskArtifactStateCacheAccess cacheAccess = new DefaultTaskArtifactStateCacheAccess(gradle, cacheRepository);
         FileSnapshotter inputFilesSnapshotter = new DefaultFileSnapshotter(new DefaultHasher());
-        FileSnapshotter outputFilesSnapshotter = new OutputFilesSnapshotter(inputFilesSnapshotter, new RandomLongIdGenerator(), cacheRepository);
-        repository = new DefaultTaskArtifactStateRepository(cacheRepository, inputFilesSnapshotter, outputFilesSnapshotter);
+        FileSnapshotter outputFilesSnapshotter = new OutputFilesSnapshotter(inputFilesSnapshotter, new RandomLongIdGenerator(), cacheAccess);
+        TaskHistoryRepository taskHistoryRepository = new CacheBackedTaskHistoryRepository(cacheAccess, new CacheBackedFileSnapshotRepository(cacheAccess));
+        repository = new DefaultTaskArtifactStateRepository(taskHistoryRepository, inputFilesSnapshotter, outputFilesSnapshotter);
     }
 
     @Test
     public void artifactsAreNotUpToDateWhenCacheIsEmpty() {
-        expectEmptyCacheLocated();
-
         TaskArtifactState state = repository.getStateFor(task());
         assertNotNull(state);
         assertFalse(state.isUpToDate());
@@ -263,7 +249,7 @@ public class DefaultTaskArtifactStateRepositoryTest {
         TaskArtifactState state = repository.getStateFor(task());
         assertFalse(state.isUpToDate());
     }
-    
+
     @Test
     public void artifactsAreNotUpToDateWhenAnyFileInInputDirChangesType() {
         execute();
@@ -310,7 +296,6 @@ public class DefaultTaskArtifactStateRepositoryTest {
 
     @Test
     public void artifactsAreNotUpToDateWhenStateHasNotBeenUpdated() {
-        expectEmptyCacheLocated();
         repository.getStateFor(task());
 
         TaskArtifactState state = repository.getStateFor(task());
@@ -321,14 +306,13 @@ public class DefaultTaskArtifactStateRepositoryTest {
     public void artifactsAreNotUpToDateWhenOutputDirWhichUsedToExistHasBeenDeleted() {
         // Output dir already exists before first execution of task
         outputDirFile.createFile();
-        expectEmptyCacheLocated();
 
         TaskInternal task1 = builder().withOutputFiles(outputDir).createsFiles(outputDirFile).task();
         TaskInternal task2 = builder().withPath("other").withOutputFiles(outputDir).createsFiles(outputDirFile2).task();
 
         TaskArtifactState state = repository.getStateFor(task1);
         assertFalse(state.isUpToDate());
-        state.update();
+        state.afterTask();
 
         outputDir.deleteDir();
 
@@ -336,7 +320,7 @@ public class DefaultTaskArtifactStateRepositoryTest {
         state = repository.getStateFor(task2);
         assertFalse(state.isUpToDate());
         task2.execute();
-        state.update();
+        state.afterTask();
 
         // Task should be out-of-date
         state = repository.getStateFor(task1);
@@ -376,18 +360,16 @@ public class DefaultTaskArtifactStateRepositoryTest {
 
     @Test
     public void hasEmptyTaskHistoryWhenTaskHasNeverBeenExecuted() {
-        expectEmptyCacheLocated();
-
         TaskArtifactState state = repository.getStateFor(task());
-        assertThat(state.getOutputFiles().getFiles(), isEmpty());
+        assertThat(state.getExecutionHistory().getOutputFiles().getFiles(), isEmpty());
     }
-    
+
     @Test
     public void hasTaskHistoryFromPreviousExecution() {
         execute();
 
         TaskArtifactState state = repository.getStateFor(task());
-        assertThat(state.getOutputFiles().getFiles(), equalTo(toLinkedSet((File) outputFile, outputDirFile, outputDirFile2)));
+        assertThat(state.getExecutionHistory().getOutputFiles().getFiles(), equalTo(toLinkedSet((File) outputFile, outputDirFile, outputDirFile2)));
     }
 
     @Test
@@ -439,13 +421,11 @@ public class DefaultTaskArtifactStateRepositoryTest {
 
         TaskArtifactState state = repository.getStateFor(task());
         assertTrue(state.isUpToDate());
-        assertThat(state.getOutputFiles().getFiles(), (Matcher) not(hasItem(otherFile)));
+        assertThat(state.getExecutionHistory().getOutputFiles().getFiles(), (Matcher) not(hasItem(otherFile)));
     }
 
     @Test
     public void considersExistingFileInOutputDirectoryWhichIsUpdatedByTheTaskAsProducedByTask() {
-        expectEmptyCacheLocated();
-        
         TestFile otherFile = outputDir.file("other").createFile();
 
         TaskInternal task = task();
@@ -455,13 +435,13 @@ public class DefaultTaskArtifactStateRepositoryTest {
         task.execute();
         otherFile.write("new content");
 
-        state.update();
+        state.afterTask();
 
         otherFile.delete();
 
         state = repository.getStateFor(task());
         assertFalse(state.isUpToDate());
-        assertThat(state.getOutputFiles().getFiles(), (Matcher) hasItem(otherFile));
+        assertThat(state.getExecutionHistory().getOutputFiles().getFiles(), hasItem((File) otherFile));
     }
 
     @Test
@@ -472,14 +452,14 @@ public class DefaultTaskArtifactStateRepositoryTest {
 
         TaskArtifactState state = repository.getStateFor(task());
         assertFalse(state.isUpToDate());
-        state.update();
+        state.afterTask();
 
         outputDirFile.write("ignore me");
 
         state = repository.getStateFor(task());
         assertTrue(state.isUpToDate());
-        assertThat(state.getOutputFiles().getFiles(), (Matcher) not(hasItem(outputDirFile)));
-        state.update();
+        assertThat(state.getExecutionHistory().getOutputFiles().getFiles(), not(hasItem((File) outputDirFile)));
+        state.afterTask();
     }
 
     @Test
@@ -496,25 +476,6 @@ public class DefaultTaskArtifactStateRepositoryTest {
         assertFalse(state.isUpToDate());
     }
 
-    @Test
-    public void artifactsAreNotUpToDateWhenTaskDoesNotProduceAnyOutputs() {
-        TaskInternal task = builder().doesNotProduceOutput().task();
-        execute(task);
-
-        TaskArtifactState state = repository.getStateFor(task);
-        assertFalse(state.isUpToDate());
-    }
-
-    @Test
-    public void taskHistoryIsEmptyWhenTaskDoesNotProduceAnyOutout() {
-        TaskInternal task = builder().doesNotProduceOutput().task();
-        execute(task);
-
-        TaskArtifactState state = repository.getStateFor(task);
-        assertFalse(state.isUpToDate());
-        assertThat(state.getOutputFiles(), isEmpty());
-    }
-
     @Test
     public void artifactsAreUpToDateWhenTaskHasNoInputFiles() {
         TaskInternal task = builder().withInputFiles().task();
@@ -544,11 +505,11 @@ public class DefaultTaskArtifactStateRepositoryTest {
 
         TaskArtifactState state = repository.getStateFor(instance1);
         assertTrue(state.isUpToDate());
-        assertThat(state.getOutputFiles().getFiles(), equalTo(toLinkedSet((File) outputDirFile)));
+        assertThat(state.getExecutionHistory().getOutputFiles().getFiles(), equalTo(toLinkedSet((File) outputDirFile)));
 
         state = repository.getStateFor(instance2);
         assertTrue(state.isUpToDate());
-        assertThat(state.getOutputFiles().getFiles(), equalTo(toLinkedSet((File) outputDirFile2)));
+        assertThat(state.getExecutionHistory().getOutputFiles().getFiles(), equalTo(toLinkedSet((File) outputDirFile2)));
     }
 
     private void execute() {
@@ -556,32 +517,13 @@ public class DefaultTaskArtifactStateRepositoryTest {
     }
 
     private void execute(TaskInternal... tasks) {
-        expectEmptyCacheLocated();
         for (TaskInternal task : tasks) {
             TaskArtifactState state = repository.getStateFor(task);
             state.isUpToDate();
             task.execute();
-            state.update();
+            state.afterTask();
         }
     }
-    
-    private void expectEmptyCacheLocated() {
-        context.checking(new Expectations(){{
-            CacheBuilder builder = context.mock(CacheBuilder.class);
-
-            one(cacheRepository).cache("taskArtifacts");
-            will(returnValue(builder));
-
-            one(builder).forObject(gradle);
-            will(returnValue(builder));
-
-            one(builder).open();
-            will(returnValue(persistentCache));
-            
-            one(persistentCache).openIndexedCache(with(notNullValue(Serializer.class)));
-            will(returnValue(new TestIndexedCache()));
-        }});
-    }
 
     private TaskInternal task() {
         return builder().task();
@@ -630,11 +572,6 @@ public class DefaultTaskArtifactStateRepositoryTest {
             return this;
         }
 
-        public TaskBuilder doesNotProduceOutput() {
-            outputs = null;
-            return this;
-        }
-
         public TaskBuilder withProperty(String name, Object value) {
             inputProperties.put(name, value);
             return this;
@@ -666,19 +603,4 @@ public class DefaultTaskArtifactStateRepositoryTest {
     public static class TaskSubType extends DefaultTask {
     }
 
-    public static class TestIndexedCache implements PersistentIndexedCache<Object, Object> {
-        Map<Object, Object> entries = new HashMap<Object, Object>();
-
-        public Object get(Object key) {
-            return entries.get(key);
-        }
-
-        public void put(Object key, Object value) {
-            entries.put(key, value);
-        }
-
-        public void remove(Object key) {
-            entries.remove(key);
-        }
-    }
 }
