@@ -16,17 +16,19 @@
 
 package org.gradle.api.internal.project.taskfactory;
 
-import org.gradle.api.*;
+import org.gradle.api.DefaultTask;
+import org.gradle.api.GradleException;
+import org.gradle.api.Task;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.internal.AbstractTask;
 import org.gradle.api.internal.TaskInternal;
 import org.gradle.api.internal.project.DefaultProject;
-import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.tasks.*;
+import org.gradle.test.fixtures.file.TestFile;
+import org.gradle.test.fixtures.file.TestNameTestDirectoryProvider;
 import org.gradle.util.GFileUtils;
 import org.gradle.util.HelperUtil;
-import org.gradle.util.TemporaryFolder;
-import org.gradle.util.TestFile;
+import org.gradle.util.ReflectionUtil;
 import org.jmock.Expectations;
 import org.jmock.integration.junit4.JMock;
 import org.jmock.integration.junit4.JUnit4Mockery;
@@ -36,12 +38,12 @@ import org.junit.Test;
 import org.junit.runner.RunWith;
 
 import java.io.File;
-import java.util.HashMap;
-import java.util.Map;
+import java.util.*;
 import java.util.concurrent.Callable;
 
-import static org.gradle.util.Matchers.*;
-import static org.gradle.util.WrapUtil.*;
+import static org.gradle.util.Matchers.isEmpty;
+import static org.gradle.util.WrapUtil.toList;
+import static org.gradle.util.WrapUtil.toSet;
 import static org.hamcrest.Matchers.*;
 import static org.junit.Assert.*;
 
@@ -52,15 +54,15 @@ public class AnnotationProcessingTaskFactoryTest {
     }};
 
     private final ITaskFactory delegate = context.mock(ITaskFactory.class);
-    private final ProjectInternal project = context.mock(ProjectInternal.class);
     private final Map args = new HashMap();
     @Rule
-    public TemporaryFolder tmpDir = new TemporaryFolder();
-    private final TestFile testDir = tmpDir.getDir();
+    public TestNameTestDirectoryProvider tmpDir = new TestNameTestDirectoryProvider();
+    private final TestFile testDir = tmpDir.getTestDirectory();
     private final File existingFile = testDir.file("file.txt").touch();
     private final File missingFile = testDir.file("missing.txt");
     private final TestFile existingDir = testDir.file("dir").createDir();
     private final File missingDir = testDir.file("missing-dir");
+    private final File missingDir2 = testDir.file("missing-dir2");
     private final AnnotationProcessingTaskFactory factory = new AnnotationProcessingTaskFactory(delegate);
 
     @Test
@@ -90,11 +92,11 @@ public class AnnotationProcessingTaskFactoryTest {
 
     private <T extends Task> T expectTaskCreated(final T task) {
         context.checking(new Expectations() {{
-            one(delegate).createTask(project, args);
+            one(delegate).createTask(args);
             will(returnValue(task));
         }});
 
-        assertThat(factory.createTask(project, args), sameInstance((Object) task));
+        assertThat(factory.createTask(args), sameInstance((Object) task));
         return task;
     }
 
@@ -141,7 +143,7 @@ public class AnnotationProcessingTaskFactoryTest {
         TaskWithInputFile task = expectTaskCreated(TaskWithInputFile.class, existingFile);
         TaskWithInputFile task2 = expectTaskCreated(TaskWithInputFile.class, missingFile);
 
-        assertThat(task.getActions().get(0), sameInstance((Action) task2.getActions().get(0)));
+        assertThat(ReflectionUtil.getProperty(task.getActions().get(0), "action"), sameInstance(ReflectionUtil.getProperty(task2.getActions().get(0), "action")));
     }
     
     @Test
@@ -207,20 +209,19 @@ public class AnnotationProcessingTaskFactoryTest {
     @Test
     public void validationActionFailsWhenInputFileNotSpecified() {
         TaskWithInputFile task = expectTaskCreated(TaskWithInputFile.class, new Object[]{null});
-        assertValidationFails(task, "Error validating task ':task': No value has been specified for property 'inputFile'.");
+        assertValidationFails(task, "No value has been specified for property 'inputFile'.");
     }
 
     @Test
     public void validationActionFailsWhenInputFileDoesNotExist() {
         TaskWithInputFile task = expectTaskCreated(TaskWithInputFile.class, missingFile);
-        assertValidationFails(task, String.format("Error validating task ':task': File '%s' specified for property 'inputFile' does not exist.",
-                task.inputFile));
+        assertValidationFails(task, String.format("File '%s' specified for property 'inputFile' does not exist.", task.inputFile));
     }
 
     @Test
     public void validationActionFailsWhenInputFileIsADirectory() {
         TaskWithInputFile task = expectTaskCreated(TaskWithInputFile.class, existingDir);
-        assertValidationFails(task, String.format("Error validating task ':task': File '%s' specified for property 'inputFile' is not a file.",
+        assertValidationFails(task, String.format("File '%s' specified for property 'inputFile' is not a file.",
                 task.inputFile));
     }
 
@@ -243,7 +244,13 @@ public class AnnotationProcessingTaskFactoryTest {
     }
 
     @Test
-    public void validationActionSucceedsWhenSpecifiedOutputFileIsNotAFile() {
+    public void validationActionSucceedsWhenSpecifiedOutputFilesIsAFile() {
+        TaskWithOutputFiles task = expectTaskCreated(TaskWithOutputFiles.class, Collections.singletonList(existingFile));
+        task.execute();
+    }
+
+    @Test
+    public void validationActionSucceedsWhenSpecifiedOutputFileDoesNotExist() {
         TaskWithOutputFile task = expectTaskCreated(TaskWithOutputFile.class, new File(testDir, "subdir/output.txt"));
 
         task.execute();
@@ -252,41 +259,100 @@ public class AnnotationProcessingTaskFactoryTest {
     }
 
     @Test
-    public void validationActionFailsWhenOutputFileNotSpecified() {
-        TaskWithOutputFile task = expectTaskCreated(TaskWithOutputFile.class, new Object[]{null});
-        assertValidationFails(task, "Error validating task ':task': No value has been specified for property 'outputFile'.");
+    public void validationActionSucceedsWhenSpecifiedOutputFilesDoesNotExist() {
+        TaskWithOutputFiles task = expectTaskCreated(TaskWithOutputFiles.class, Arrays.asList(new File(testDir, "subdir/output.txt"), new File(testDir, "subdir2/output.txt")));
+
+        task.execute();
+
+        assertTrue(new File(testDir, "subdir").isDirectory());
+        assertTrue(new File(testDir, "subdir2").isDirectory());
     }
 
+    @Test
+    public void validationActionSucceedsWhenOptionalOutputFileNotSpecified() {
+        TaskWithOptionalOutputFile task = expectTaskCreated(TaskWithOptionalOutputFile.class);
+        task.execute();
+    }
+
+    @Test
+    public void validationActionSucceedsWhenOptionalOutputFilesNotSpecified() {
+        TaskWithOptionalOutputFiles task = expectTaskCreated(TaskWithOptionalOutputFiles.class);
+        task.execute();
+    }
+    
+    @Test
+    public void validationActionFailsWhenOutputFileNotSpecified() {
+        TaskWithOutputFile task = expectTaskCreated(TaskWithOutputFile.class, new Object[]{null});
+        assertValidationFails(task, "No value has been specified for property 'outputFile'.");
+    }
+
+    @Test
+    public void validationActionFailsWhenOutputFilesNotSpecified() {
+        TaskWithOutputFiles task = expectTaskCreated(TaskWithOutputFiles.class, new Object[]{null});
+        assertValidationFails(task, "No value has been specified for property 'outputFiles'.");
+    }
+    
     @Test
     public void validationActionFailsWhenSpecifiedOutputFileIsADirectory() {
         TaskWithOutputFile task = expectTaskCreated(TaskWithOutputFile.class, existingDir);
         assertValidationFails(task, String.format(
-                "Error validating task ':task': Cannot write to file '%s' specified for property 'outputFile' as it is a directory.",
+                "Cannot write to file '%s' specified for property 'outputFile' as it is a directory.",
                 task.outputFile));
     }
 
+    @Test
+    public void validationActionFailsWhenSpecifiedOutputFilesIsADirectory() {
+        TaskWithOutputFiles task = expectTaskCreated(TaskWithOutputFiles.class, Collections.singletonList(existingDir));
+        assertValidationFails(task, String.format(
+                "Cannot write to file '%s' specified for property 'outputFiles' as it is a directory.",
+                task.outputFiles.get(0)));
+    }
+    
     @Test
     public void validationActionFailsWhenSpecifiedOutputFileParentIsAFile() {
         TaskWithOutputFile task = expectTaskCreated(TaskWithOutputFile.class, new File(testDir, "subdir/output.txt"));
         GFileUtils.touch(task.outputFile.getParentFile());
 
-        assertValidationFails(task, String.format(
-                "Error validating task ':task': Cannot create parent directory '%s' of file specified for property 'outputFile'.",
-                task.outputFile.getParentFile()));
+        assertValidationFails(task, String.format("Cannot write to file '%s' specified for property 'outputFile', as ancestor '%s' is not a directory.",
+                task.getOutputFile(), task.outputFile.getParentFile()));
     }
 
+    @Test
+    public void validationActionFailsWhenSpecifiedOutputFilesParentIsAFile() {
+        TaskWithOutputFiles task = expectTaskCreated(TaskWithOutputFiles.class, Collections.singletonList(new File(testDir, "subdir/output.txt")));
+        GFileUtils.touch(task.outputFiles.get(0).getParentFile());
+
+        assertValidationFails(task, String.format("Cannot write to file '%s' specified for property 'outputFiles', as ancestor '%s' is not a directory.",
+                task.outputFiles.get(0), task.outputFiles.get(0).getParentFile()));
+    }
+    
     @Test
     public void registersSpecifiedOutputFile() {
         TaskWithOutputFile task = expectTaskCreated(TaskWithOutputFile.class, existingFile);
         assertThat(task.getOutputs().getFiles().getFiles(), equalTo(toSet(existingFile)));
     }
 
+    @Test
+    public void registersSpecifiedOutputFiles() {
+        TaskWithOutputFiles task = expectTaskCreated(TaskWithOutputFiles.class, Collections.singletonList(existingFile));
+        assertThat(task.getOutputs().getFiles().getFiles(), equalTo(toSet(existingFile)));
+    }
+
     @Test
     public void doesNotRegisterOutputFileWhenNoneSpecified() {
         TaskWithOutputFile task = expectTaskCreated(TaskWithOutputFile.class, new Object[]{null});
         assertThat(task.getOutputs().getFiles().getFiles(), isEmpty());
     }
 
+    @Test
+    public void doesNotRegisterOutputFilesWhenNoneSpecified() {
+        TaskWithOutputFiles task = expectTaskCreated(TaskWithOutputFiles.class, new Object[]{null});
+        assertThat(task.getOutputs().getFiles().getFiles(), isEmpty());
+
+        task = expectTaskCreated(TaskWithOutputFiles.class, Collections.<File>emptyList());
+        assertThat(task.getOutputs().getFiles().getFiles(), isEmpty());
+    }
+
     @Test
     public void validationActionSucceedsWhenInputFilesSpecified() {
         TaskWithInputFiles task = expectTaskCreated(TaskWithInputFiles.class, toList(testDir));
@@ -296,7 +362,7 @@ public class AnnotationProcessingTaskFactoryTest {
     @Test
     public void validationActionFailsWhenInputFilesNotSpecified() {
         TaskWithInputFiles task = expectTaskCreated(TaskWithInputFiles.class, new Object[]{null});
-        assertValidationFails(task, "Error validating task ':task': No value has been specified for property 'input'.");
+        assertValidationFails(task, "No value has been specified for property 'input'.");
     }
 
     @Test
@@ -315,11 +381,11 @@ public class AnnotationProcessingTaskFactoryTest {
     public void skipsTaskWhenInputFileCollectionIsEmpty() {
         final FileCollection inputFiles = context.mock(FileCollection.class);
         context.checking(new Expectations() {{
-            one(inputFiles).stopExecutionIfEmpty();
-            will(throwException(new StopExecutionException()));
+            one(inputFiles).isEmpty();
+            will(returnValue(true));
         }});
 
-        TaskWithInputFiles task = expectTaskCreated(BrokenTaskWithInputFiles.class, inputFiles);
+        BrokenTaskWithInputFiles task = expectTaskCreated(BrokenTaskWithInputFiles.class, inputFiles);
 
         task.execute();
     }
@@ -332,46 +398,107 @@ public class AnnotationProcessingTaskFactoryTest {
         assertTrue(task.outputDir.isDirectory());
     }
 
+    @Test
+    public void validationActionSucceedsWhenSpecifiedOutputDirectoriesDoesNotExist() {
+        TaskWithOutputDirs task = expectTaskCreated(TaskWithOutputDirs.class, Collections.singletonList(missingDir));
+        task.execute();
+
+        assertTrue(task.outputDirs.get(0).isDirectory());
+    }
+
     @Test
     public void validationActionSucceedsWhenSpecifiedOutputDirectoryIsDirectory() {
         TaskWithOutputDir task = expectTaskCreated(TaskWithOutputDir.class, existingDir);
         task.execute();
     }
 
+    @Test
+    public void validationActionSucceedsWhenSpecifiedOutputDirectoriesAreDirectories() {
+        TaskWithOutputDirs task = expectTaskCreated(TaskWithOutputDirs.class, Collections.singletonList(existingDir));
+        task.execute();
+    }
+    
+    @Test
+    public void validationActionSucceedsWhenOptionalOutputDirectoryNotSpecified() {
+        TaskWithOptionalOutputDir task = expectTaskCreated(TaskWithOptionalOutputDir.class);
+        task.execute();
+    }
+
+    @Test
+    public void validationActionSucceedsWhenOptionalOutputDirectoriesNotSpecified() {
+        TaskWithOptionalOutputDirs task = expectTaskCreated(TaskWithOptionalOutputDirs.class);
+        task.execute();
+    }
+
     @Test
     public void validationActionFailsWhenOutputDirectoryNotSpecified() {
         TaskWithOutputDir task = expectTaskCreated(TaskWithOutputDir.class, new Object[]{null});
-        assertValidationFails(task, "Error validating task ':task': No value has been specified for property 'outputDir'.");
+        assertValidationFails(task, "No value has been specified for property 'outputDir'.");
+    }
+
+    @Test
+    public void validationActionFailsWhenOutputDirectoriesNotSpecified() {
+        TaskWithOutputDirs task = expectTaskCreated(TaskWithOutputDirs.class, new Object[]{null});
+        assertValidationFails(task, "No value has been specified for property 'outputDirs'.");
     }
 
     @Test
     public void validationActionFailsWhenOutputDirectoryIsAFile() {
         TaskWithOutputDir task = expectTaskCreated(TaskWithOutputDir.class, existingFile);
-        assertValidationFails(task, String.format("Error validating task ':task': Cannot create directory '%s' specified for property 'outputDir'.",
+        assertValidationFails(task, String.format("Directory '%s' specified for property 'outputDir' is not a directory.",
                 task.outputDir));
     }
 
+    @Test
+    public void validationActionFailsWhenOutputDirectoriesIsAFile() {
+        TaskWithOutputDirs task = expectTaskCreated(TaskWithOutputDirs.class, Collections.singletonList(existingFile));
+        assertValidationFails(task, String.format("Directory '%s' specified for property 'outputDirs' is not a directory.",
+                task.outputDirs.get(0)));
+    }
+    
     @Test
     public void validationActionFailsWhenParentOfOutputDirectoryIsAFile() {
         TaskWithOutputDir task = expectTaskCreated(TaskWithOutputDir.class, new File(testDir, "subdir/output"));
         GFileUtils.touch(task.outputDir.getParentFile());
 
-        assertValidationFails(task, String.format("Error validating task ':task': Cannot create directory '%s' specified for property 'outputDir'.",
-                task.outputDir));
+        assertValidationFails(task, String.format("Cannot write to directory '%s' specified for property 'outputDir', as ancestor '%s' is not a directory.", task.outputDir, task.outputDir.getParentFile()));
     }
 
+    @Test
+    public void validationActionFailsWhenParentOfOutputDirectoriesIsAFile() {
+        TaskWithOutputDirs task = expectTaskCreated(TaskWithOutputDirs.class, Collections.singletonList(new File(testDir, "subdir/output")));
+        GFileUtils.touch(task.outputDirs.get(0).getParentFile());
+
+        assertValidationFails(task, String.format("Cannot write to directory '%s' specified for property 'outputDirs', as ancestor '%s' is not a directory.", task.outputDirs.get(0), task.outputDirs.get(0).getParentFile()));
+    }
+    
     @Test
     public void registersSpecifiedOutputDirectory() {
         TaskWithOutputDir task = expectTaskCreated(TaskWithOutputDir.class, missingDir);
         assertThat(task.getOutputs().getFiles().getFiles(), equalTo(toSet(missingDir)));
     }
 
+    @Test
+    public void registersSpecifiedOutputDirectories() {
+        TaskWithOutputDirs task = expectTaskCreated(TaskWithOutputDirs.class, Arrays.<File>asList(missingDir, missingDir2));
+        assertThat(task.getOutputs().getFiles().getFiles(), equalTo(toSet(missingDir, missingDir2)));
+    }
+    
     @Test
     public void doesNotRegisterOutputDirectoryWhenNoneSpecified() {
         TaskWithOutputDir task = expectTaskCreated(TaskWithOutputDir.class, new Object[]{null});
         assertThat(task.getOutputs().getFiles().getFiles(), isEmpty());
     }
 
+    @Test
+    public void doesNotRegisterOutputDirectoriesWhenNoneSpecified() {
+        TaskWithOutputDirs task = expectTaskCreated(TaskWithOutputDirs.class, new Object[]{null});
+        assertThat(task.getOutputs().getFiles().getFiles(), isEmpty());
+
+        task = expectTaskCreated(TaskWithOutputDirs.class, Collections.<File>emptyList());
+        assertThat(task.getOutputs().getFiles().getFiles(), isEmpty());
+    }
+
     @Test
     public void validationActionSucceedsWhenSpecifiedInputDirectoryIsDirectory() {
         TaskWithInputDir task = expectTaskCreated(TaskWithInputDir.class, existingDir);
@@ -381,13 +508,13 @@ public class AnnotationProcessingTaskFactoryTest {
     @Test
     public void validationActionFailsWhenInputDirectoryNotSpecified() {
         TaskWithInputDir task = expectTaskCreated(TaskWithInputDir.class, new Object[]{null});
-        assertValidationFails(task, "Error validating task ':task': No value has been specified for property 'inputDir'.");
+        assertValidationFails(task, "No value has been specified for property 'inputDir'.");
     }
     
     @Test
     public void validationActionFailsWhenInputDirectoryDoesNotExist() {
         TaskWithInputDir task = expectTaskCreated(TaskWithInputDir.class, missingDir);
-        assertValidationFails(task, String.format("Error validating task ':task': Directory '%s' specified for property 'inputDir' does not exist.",
+        assertValidationFails(task, String.format("Directory '%s' specified for property 'inputDir' does not exist.",
                 task.inputDir));
     }
 
@@ -397,7 +524,7 @@ public class AnnotationProcessingTaskFactoryTest {
         GFileUtils.touch(task.inputDir);
 
         assertValidationFails(task, String.format(
-                "Error validating task ':task': Directory '%s' specified for property 'inputDir' is not a directory.", task.inputDir));
+                "Directory '%s' specified for property 'inputDir' is not a directory.", task.inputDir));
     }
 
     @Test
@@ -419,12 +546,6 @@ public class AnnotationProcessingTaskFactoryTest {
         task.execute();
     }
 
-    @Test
-    public void skipsTaskWhenInputDirectoryIsDoesNotExistAndSkipWhenEmpty() {
-        TaskWithInputDir task = expectTaskCreated(BrokenTaskWithInputDir.class, missingDir);
-        task.execute();
-    }
-
     @Test
     public void validationActionSucceedsWhenInputValueSpecified() {
         TaskWithInput task = expectTaskCreated(TaskWithInput.class, "value");
@@ -434,7 +555,7 @@ public class AnnotationProcessingTaskFactoryTest {
     @Test
     public void validationActionFailsWhenInputValueNotSpecified() {
         TaskWithInput task = expectTaskCreated(TaskWithInput.class, new Object[]{null});
-        assertValidationFails(task, "Error validating task ':task': No value has been specified for property 'inputValue'.");
+        assertValidationFails(task, "No value has been specified for property 'inputValue'.");
     }
 
     @Test
@@ -452,7 +573,13 @@ public class AnnotationProcessingTaskFactoryTest {
     @Test
     public void validatesNestedBeans() {
         TaskWithNestedBean task = expectTaskCreated(TaskWithNestedBean.class, new Object[]{null});
-        assertValidationFails(task, "Error validating task ':task': No value has been specified for property 'bean.inputFile'.");
+        assertValidationFails(task, "No value has been specified for property 'bean.inputFile'.");
+    }
+
+    @Test
+    public void validatesNestedBeansWithPrivateType() {
+        TaskWithNestedBeanWithPrivateClass task = expectTaskCreated(TaskWithNestedBeanWithPrivateClass.class, new Object[]{existingFile, null});
+        assertValidationFails(task, "No value has been specified for property 'bean.inputFile'.");
     }
 
     @Test
@@ -461,17 +588,36 @@ public class AnnotationProcessingTaskFactoryTest {
         assertThat(task.getInputs().getProperties().get("bean.class"), equalTo((Object) Bean.class.getName()));
     }
 
+    @Test
+    public void registersInputPropertyForNestedBeanClassWithPrivateType() {
+        TaskWithNestedBeanWithPrivateClass task = expectTaskCreated(TaskWithNestedBeanWithPrivateClass.class, new Object[]{null, null});
+        assertThat(task.getInputs().getProperties().get("bean.class"), equalTo((Object) Bean2.class.getName()));
+    }
+
     @Test
     public void doesNotRegisterInputPropertyWhenNestedBeanIsNull() {
         TaskWithOptionalNestedBean task = expectTaskCreated(TaskWithOptionalNestedBean.class);
         assertThat(task.getInputs().getProperties().get("bean.class"), nullValue());
     }
 
+    @Test
+    public void doesNotRegisterInputPropertyWhenNestedBeanWithPrivateTypeIsNull() {
+        TaskWithOptionalNestedBeanWithPrivateType task = expectTaskCreated(TaskWithOptionalNestedBeanWithPrivateType.class);
+        assertThat(task.getInputs().getProperties().get("bean.class"), nullValue());
+    }
+
     @Test
     public void validationFailsWhenNestedBeanIsNull() {
         TaskWithNestedBean task = expectTaskCreated(TaskWithNestedBean.class, new Object[]{null});
         task.bean = null;
-        assertValidationFails(task, "Error validating task ':task': No value has been specified for property 'bean'.");
+        assertValidationFails(task, "No value has been specified for property 'bean'.");
+    }
+
+    @Test
+    public void validationFailsWhenNestedBeanWithPrivateTypeIsNull() {
+        TaskWithNestedBeanWithPrivateClass task = expectTaskCreated(TaskWithNestedBeanWithPrivateClass.class, new Object[]{null, null});
+        task.bean = null;
+        assertValidationFails(task, "No value has been specified for property 'bean'.");
     }
 
     @Test
@@ -480,19 +626,41 @@ public class AnnotationProcessingTaskFactoryTest {
         task.execute();
     }
 
+    @Test
+    public void validationSucceedsWhenNestedBeanWithPrivateTypeIsNullAndMarkedOptional() {
+        TaskWithOptionalNestedBeanWithPrivateType task = expectTaskCreated(TaskWithOptionalNestedBeanWithPrivateType.class);
+        task.execute();
+    }
+
     @Test
     public void canAttachAnnotationToGroovyProperty() {
         InputFileTask task = expectTaskCreated(InputFileTask.class);
-        assertValidationFails(task, "Error validating task ':task': No value has been specified for property 'srcFile'.");
+        assertValidationFails(task, "No value has been specified for property 'srcFile'.");
     }
-    
-    private void assertValidationFails(TaskInternal task, String expectedErrorMessage) {
+
+    @Test
+    public void validationFailureListsViolationsForAllProperties() {
+        TaskWithMultipleProperties task = expectTaskCreated(TaskWithMultipleProperties.class, new Object[]{null});
+        assertValidationFails(task,
+                "No value has been specified for property 'outputFile'.",
+                "No value has been specified for property 'bean.inputFile'.");
+    }
+
+    private void assertValidationFails(TaskInternal task, String... expectedErrorMessages) {
         try {
             task.execute();
             fail();
-        } catch (GradleException e) {
-            assertThat(e.getCause(), instanceOf(InvalidUserDataException.class));
-            assertThat(e.getCause().getMessage(), equalTo(expectedErrorMessage));
+        } catch (TaskValidationException e) {
+            if (expectedErrorMessages.length == 1) {
+                assertThat(e.getMessage(), containsString("A problem was found with the configuration of " + task));
+            } else {
+                assertThat(e.getMessage(), containsString("Some problems were found with the configuration of " + task));
+            }
+            HashSet<String> actualMessages = new HashSet<String>();
+            for (Throwable cause : e.getCauses()) {
+                actualMessages.add(cause.getMessage());
+            }
+            assertThat(actualMessages, equalTo(new HashSet<String>(Arrays.asList(expectedErrorMessages))));
         }
     }
 
@@ -639,6 +807,33 @@ public class AnnotationProcessingTaskFactoryTest {
         }
     }
 
+    public static class TaskWithOutputFiles extends DefaultTask {
+        List<File> outputFiles;
+
+        public TaskWithOutputFiles(List<File> outputFiles) {
+            this.outputFiles = outputFiles;
+        }
+
+        @OutputFiles
+        public List<File> getOutputFiles() {
+            return outputFiles;
+        }
+    }
+    
+    public static class TaskWithOptionalOutputFile extends DefaultTask {
+        @OutputFile @Optional
+        public File getOutputFile() {
+            return null;
+        }
+    }
+
+    public static class TaskWithOptionalOutputFiles extends DefaultTask {
+        @OutputFiles @Optional
+        public List<File> getOutputFiles() {
+            return null;
+        }
+    }
+
     public static class TaskWithOutputDir extends DefaultTask {
         File outputDir;
 
@@ -652,6 +847,33 @@ public class AnnotationProcessingTaskFactoryTest {
         }
     }
 
+    public static class TaskWithOutputDirs extends DefaultTask {
+        List<File> outputDirs;
+
+        public TaskWithOutputDirs(List<File> outputDirs) {
+            this.outputDirs = outputDirs;
+        }
+
+        @OutputDirectories
+        public List<File> getOutputDirs() {
+            return outputDirs;
+        }
+    }
+    
+    public static class TaskWithOptionalOutputDir extends DefaultTask {
+        @OutputDirectory @Optional
+        public File getOutputDir() {
+            return null;
+        }
+    }
+
+    public static class TaskWithOptionalOutputDirs extends DefaultTask {
+        @OutputDirectories @Optional
+        public File getOutputDirs() {
+            return null;
+        }
+    }
+
     public static class TaskWithInputFiles extends DefaultTask {
         Iterable<? extends File> input;
 
@@ -659,7 +881,7 @@ public class AnnotationProcessingTaskFactoryTest {
             this.input = input;
         }
 
-        @InputFiles @SkipWhenEmpty
+        @InputFiles
         public Iterable<? extends File> getInput() {
             return input;
         }
@@ -670,6 +892,11 @@ public class AnnotationProcessingTaskFactoryTest {
             super(input);
         }
 
+        @InputFiles @SkipWhenEmpty
+        public Iterable<? extends File> getInput() {
+            return input;
+        }
+
         @TaskAction
         public void doStuff() {
             fail();
@@ -696,6 +923,32 @@ public class AnnotationProcessingTaskFactoryTest {
         }
     }
 
+    
+    public static class TaskWithNestedBeanWithPrivateClass extends DefaultTask {
+        Bean2 bean = new Bean2();
+
+        public TaskWithNestedBeanWithPrivateClass(File inputFile, File inputFile2) {
+            bean.inputFile = inputFile;
+            bean.inputFile2 = inputFile2;
+        }
+
+        @Nested
+        public Bean getBean() {
+            return bean;
+        }
+    }
+    
+    public static class TaskWithMultipleProperties extends TaskWithNestedBean {
+        public TaskWithMultipleProperties(File inputFile) {
+            super(inputFile);
+        }
+
+        @OutputFile
+        public File getOutputFile() {
+            return bean.getInputFile();
+        }
+    }
+
     public static class TaskWithOptionalNestedBean extends DefaultTask {
         @Nested @Optional
         public Bean getBean() {
@@ -703,6 +956,15 @@ public class AnnotationProcessingTaskFactoryTest {
         }
     }
 
+    public static class TaskWithOptionalNestedBeanWithPrivateType extends DefaultTask {
+        Bean2 bean = new Bean2();
+
+        @Nested @Optional
+        public Bean getBean() {
+            return null;
+        }
+    }
+
     public static class Bean {
         @InputFile
         File inputFile;
@@ -711,4 +973,13 @@ public class AnnotationProcessingTaskFactoryTest {
             return inputFile;
         }
     }
+
+    public static class Bean2 extends Bean {
+        @InputFile
+        File inputFile2;
+
+        public File getInputFile() {
+            return inputFile2;
+        }
+    }
 }
