@@ -15,17 +15,8 @@
  */
 package org.gradle.api.internal.tasks.compile;
 
-import org.gradle.api.file.FileCollection;
 import org.gradle.api.tasks.WorkResult;
 
-import java.io.File;
-
-public interface Compiler {
-    void setSource(FileCollection source);
-
-    void setDestinationDir(File destinationDir);
-
-    void setClasspath(Iterable<File> classpath);
-
-    WorkResult execute();
+public interface Compiler<T extends CompileSpec> {
+    WorkResult execute(T spec);
 }
